# -*- coding: utf-8 -*-
"""
This illustrates how to use importNewProbe.
Make sure that importNewProbe.py is in your same directory or path

@author: tim@ema3d.com
"""
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from scipy import fft, arange
from scipy.interpolate import interp1d
#Read a new probe file into a numpy array using a Pandas dataframe for speed
def readFile(filename):
    print('Reading the data file from disk: ', filename)
    wave = pd.read_csv(filename,delim_whitespace=True,header=None).values.flatten()
    joe = wave[np.logical_not(np.isnan(wave))]
    return joe   
#Single-sided Fourier transform spectrum (Complex)

def make_fftx(tdx,tdy):
    N=len(tdx)
    dx=tdx[1]-tdx[0]
    #do the fft and normalize
    fdx=np.linspace(0.0,1.0/(2.0*dx),N//2)
    
    return fdx

def make_ffty(tdx,tdy):
    N=len(tdx)
    #do the fft and normalize
    fdy=2.0/N*np.abs(fft(tdy)[:N//2])
    
    return fdy
def timfft(time,funct):
    frequency1 = make_fftx(time,funct)
    functFFT = make_ffty(time,funct)
    return frequency1, functFFT 
#FFT of a single point
def fft_single_point(npr, num_steps):
    timer = npr[np.arange(0, npr.size, num_steps)]
    ExTD = npr[np.arange(1, npr.size, num_steps)]
    EyTD = npr[np.arange(2, npr.size, num_steps)]
    EzTD = npr[np.arange(3, npr.size, num_steps)]
    (freq, Ex) = timfft(timer, ExTD)
    (freq, Ey) = timfft(timer, EyTD)
    (freq, Ez) = timfft(timer, EzTD)
    return freq, Ex, Ey, Ez, timer, ExTD, EyTD, EzTD    
def bwa(freq,wave,npct):
    #Determine the cutoff frequency index
    print("Performing bandwidth average of wave")
    df = freq[1]-freq[0] #determine frequency spacing
    nFmax = freq.size
    wavebwa = np.zeros(nFmax)
    for i in range(nFmax):
        valf = freq[i] *  npct / 200; #change from percent and /2 for each side
        npc = int(valf/df) + 1
        nst = i - npc
        nnd = i + npc
        if nst < 1:
            nst = 1
        if nnd > nst:
            wavebwa[i] = np.mean(wave[nst:nnd])
    return wavebwa
def fft_antenna_aperture(npr, num_steps, gaussFFT):
    timer = npr[np.arange(0, npr.size, num_steps)]
    ExTD = npr[np.arange(1, npr.size, num_steps)]
    (freq, Ex) = timfft(timer, ExTD)
    ExBWA = bwa(freq, np.abs(Ex), 5)  
    minTFx = -20 * np.log10(ExBWA / gaussFFT)
    maxTFx = -20 * np.log10(ExBWA / gaussFFT)
    sumTFx = -20 * np.log10(ExBWA / gaussFFT)    
    print('Processing FFT of measurement point progress: ', end='')
    for iter in range(1,125):
        ExTD = npr[np.arange(3 * iter + 1, npr.size, num_steps)]
        (freq, Ex) = timfft(timer, ExTD)
        ExBWA = bwa(freq, np.abs(Ex), 5)
        TFx = -20 * np.log10(ExBWA / gaussFFT)
        print('-', end='')      
        maxTFx = np.maximum(maxTFx, TFx)
        minTFx = np.minimum(minTFx, TFx)
        sumTFx += TFx
    print('FFT complete')
    meanTFx = sumTFx/125
    return meanTFx, maxTFx, minTFx
def read_2col(filename):
    print("Reading the data file from disk: ", filename)
    df2 = pd.read_csv(filename,delim_whitespace=True,names=["time","field"])
    time = np.array(df2['time'])
    field = np.array(df2['field'])
    return time, field
def matchTimeSpacingLength(targTime, sourTime, wave):
    print("Interpolating and zero padding so the two time domain waves are the same length")
    #First interpolate so that the source wave matches the spacing of the target wave
    for iter in range(len(targTime)):
        if targTime[iter] > sourTime[-1]:
            break
    overlap = iter - 1
    sourTimeInterp = targTime
    fwave = interp1d(sourTime,wave)
    waveInterp = fwave(sourTimeInterp[0:overlap])
    #Now zero fill the target wave so that they have the same length
    if waveInterp.size < targTime.size:
        waveInterpFilled = np.pad(waveInterp, (0,targTime.size - waveInterp.size), 'constant')
    else:
        waveInterpFilled = waveInterp
    return waveInterpFilled
    

#Globals
di = 'C:/Users/Sean/Documents/F-16_High_Frequency_Demo/18/'
num_steps = 375 + 1
filer1 = di + 'F-16_18GHz_Demo_cfnprnumertime1' + '.dat' #Canopy
filer2 = di + 'F-16_18GHz_Demo_cfnprnumertime2' + '.dat' #Canopy
filer3 = di + 'F-16_18GHz_Demo_cfnprnumertime3' + '.dat' #Canopy
sourcer = di + 'gauss' + '.dat'


#Read the data
npr1 = readFile(filer1)
npr2 = readFile(filer2)
npr3 = readFile(filer3)



#Make an FFT of a single point
freq1, Ex1, Ey1, Ez1, timer, ExTD, EyTD, EzTD = fft_single_point(npr1, num_steps)
freq2, Ex2, Ey2, Ez2, timer2, ExTD2, EyTD2, EzTD2 = fft_single_point(npr2, num_steps)
freq3, Ex3, Ey3, Ez3, timer3, ExTD3, EyTD3, EzTD3 = fft_single_point(npr3, num_steps)

#Read the source data
(gaussTime, gaussField) = read_2col(sourcer)

#Interpolate source data to the result data
gaussTimeInterp = matchTimeSpacingLength(timer, gaussTime, gaussField)

#Take and FFT of source data
(gaussFreq, gaussFFTimag) = timfft(timer, gaussTimeInterp)
gaussFFT = np.abs(gaussFFTimag)


#Make an FFT of a frequency average
(MeanTFx1, MaxTFx1, MinTFx1) = fft_antenna_aperture(npr1, num_steps, gaussFFT)
(MeanTFx2, MaxTFx2, MinTFx2) = fft_antenna_aperture(npr2, num_steps, gaussFFT)
(MeanTFx3, MaxTFx3, MinTFx3) = fft_antenna_aperture(npr3, num_steps, gaussFFT)


#Plot the data
plt.figure(0)
plt.semilogx(freq1, MaxTFx1,'k', freq1, MinTFx1,'k', freq1, MeanTFx1,'b', linewidth =2)
plt.legend(('Lower bound simulation','Upper bound simulation', 'Mean simulation'))
plt.title('F16 Canopy Shielding Effectivness')
plt.xlabel('Frequency (Hz)')
plt.ylabel('SE (dB)')
plt.fill_between(freq1, MaxTFx1, MinTFx1,facecolor = 'grey')
plt.xlim(5e8,18e9)
plt.ylim(0,50)

plt.figure(1)
plt.semilogx(freq2, MaxTFx2,'k', freq2, MinTFx2,'k', freq2, MeanTFx2,'b', linewidth =2)
plt.legend(('Lower bound simulation','Upper bound simulation', 'Mean simulation'))
plt.title('F16 Avionics Bay Shielding Effectivness')
plt.xlabel('Frequency (Hz)')
plt.ylabel('SE (dB)')
plt.fill_between(freq2, MaxTFx2, MinTFx2,facecolor = 'grey')
plt.xlim(5e8,18e9)
plt.ylim(0,50)

plt.figure(2)
plt.semilogx(freq3, MaxTFx3,'k', freq3, MinTFx3,'k', freq3, MeanTFx3,'b', linewidth =2)
plt.legend(('Lower bound simulation','Upper bound simulation', 'Mean simulation'))
plt.title('F16 Control Panel Shielding Effectivness')
plt.xlabel('Frequency (Hz)')
plt.ylabel('SE (dB)')
plt.fill_between(freq3, MaxTFx3, MinTFx3,facecolor = 'grey')
plt.xlim(5e8,18e9)
plt.ylim(0,50)








